// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "LoaderForSceneItems.hpp"
#include "asset/loader/TextureLoader.hpp"
#include "asset/loader/PrefabLoader.hpp"

using namespace Dawn;

LoaderForSceneItems::LoaderForSceneItems(
  const std::shared_ptr<AssetManager> assetManager,
  const std::string name
) : AssetLoader(assetManager, name) {
}

void LoaderForSceneItems::setupDependencies() {
  // Begin loading dependencies.
  auto &data = this->jsonLoader->data;
  if(data.contains("assets")) {
    for(auto &asset : data["assets"].items()) {
      auto &assetName = asset.key();
      auto &assetData = asset.value();
      assertTrue(assetData.contains("type"), "Asset missing type");
      assertTrue(assetData.contains("path"), "Asset missing path");
      auto type = assetData["type"].get<std::string>();
      auto path = assetData["path"].get<std::string>();

      std::shared_ptr<AssetLoader> loader;
      if(type == "texture") {
        loader = getAssetManager()->get<TextureLoader>(path);

      } else if(type == "json") {
        loader = getAssetManager()->get<JSONLoader>(path);

      } else if(type == "prefab") {
        loader = getAssetManager()->get<PrefabLoader>(path);

      } else {
        assertUnreachable("Unknown asset type: %s", type.c_str());
      }
      
      assertMapNotHasKey(
        ctx.assets,
        assetName,
        "Asset already exists: %s", assetName.c_str()
      );

      ctx.assets[assetName] = loader;
    }
  }
}

LoaderForSceneItems::~LoaderForSceneItems() {
  
}